/**
 * Create a configuration placeholder from the provided filepath.
 */
exports.create = function config(filepath) {
	
	var mConfig = require(filepath);
	var instance = {};
	
	/**
	 * Return the configuration based on a `path`, or `defaultValue` if no match.
	 *  -example 
	 *		path:"hello.world" will return `mConfig['hello']['world']`
	 *
	 * @param {String} path
	 * @param {Mixed} defaultValue
	 * @return {Mixed}
	 */
	instance.get = function(path, defaultValue) {
		if (!path) {
			return mConfig;
		}

		var segments = path.split(".");
		var pointer = mConfig;

		while (segments.length > 0) {
			var index = segments.shift();
			if (!pointer[index]) {
				return defaultValue;
			}
			pointer = pointer[index];
		}

		return pointer;
	};

	instance.set = function(path, value) {
		var segments = path.split(".");
		var lastIndex = segments.pop();
		var pointer = mConfig;
		while (segments.length > 0) {
			var index = segments.shift();
			if (!pointer[index]) {
				pointer[index] = {};
			}
			pointer = pointer[index];
		}

		pointer[lastIndex] = value;
		return pointer;
	};
	
	return instance;
}
