var Mongolian = require("mongolian");

/**
 * Service locator, to provide easy access to system services.
 * This is intented to be used as a singleton but it is possible to create multiple locators.
 */
exports.create = function locator(application) {
	var mDatabase;
	var mHttpServer;
	var mSocketIO;
	var instance = {};
	
	/**
	 * Lazy loader for a mongolian singleton instance.
	 * @return {mongolian}
	 */
	instance.getDatabase = function() {
		if (!mDatabase) {
			var connection = application.config.get('mongodb.host') + ':' + application.config.get('mongodb.port');
			var databaseServer = new Mongolian(connection);
			
			mDatabase = databaseServer.db(application.config.get('mongodb.database'));
		}
		
		return mDatabase;
	};

	instance.setHttpServer = function(httpServer) {
		mHttpServer = httpServer;
	};
	
	instance.getHttpServer = function() {
		return mHttpServer;
	};
	
	instance.getSocketIO = function() {
		return mSocketIO;
	};
	
	instance.setSocketIO = function(socketIO) {
		mSocketIO = socketIO;
	}
	
	return instance;
};